@@ -6,9 +6,9 @@ from octopus_python_client.common import config, get_one_type_save, get_single_i
     update_single_item_save, create_single_item_from_local_file, item_type_deployment_processes, \
     clone_single_item_from_remote_item, delete_single_item_by_name_or_id, get_child_item_save, \
     update_child_item_from_local_save, clone_child_item_from_another_parent_save, get_types_save, \
-    item_types_only_outer_space, item_types_inside_space, deployment_process_id_key, steps_key, \
+    outer_space_download_types, inside_space_download_types, deployment_process_id_key, steps_key, \
     merge_single_item_save, delete_one_type, api_key_key, octopus_endpoint_key, octopus_name_key, user_name_key, \
-    password_key, double_hyphen, verify_space, get_spaces_save, get_task_status, wait_task
+    password_key, double_hyphen, verify_space, get_spaces_save, get_task_status, wait_task, delete_types
 from octopus_python_client.migration import Migration
 from octopus_python_client.processes import clone_process_step, delete_process_step
 from octopus_python_client.projects import clone_project, delete_project, get_project, project_update_variable_sets
@@ -28,6 +28,8 @@ class Actions:
     action_get_types = "get_types"  # get all types
     action_get_type = "get_type"  # get all items under one type
     action_delete_type = "delete_type"  # delete all items under one type
+    # delete all items under specified types (if no types specified, delete all cloneable types in reverse order)
+    action_delete_types = "delete_types"
     action_get = "get"  # get one item
     action_update = "update"  # update one item
     action_update_merge = "update_merge"  # update one item and merge the existing sub-items
@@ -80,10 +82,11 @@ def _parse_args():
                              'regardless whether they are above Spaces; if (not item_types) and octopus_space_id, '
                              'get all item types below octopus_space_id; '
                              'list like "accounts,actiontemplates,artifacts" is also accepted; '
-                             'item types above Spaces: ' + ", ".join(item_types_only_outer_space) +
-                             "; \nitem types above and under Spaces: " + ", ".join(item_types_inside_space))
+                             'item types above Spaces: ' + ", ".join(outer_space_download_types) +
+                             "; \nitem types above and under Spaces: " + ", ".join(inside_space_download_types))
     parser.add_argument("-tp", "--item_type", help="one of item types above Spaces: " + ", ".join(
-        item_types_only_outer_space) + "; \nitem types above and under Spaces: " + ", ".join(item_types_inside_space))
+        outer_space_download_types) + "; \nitem types above and under Spaces: " + ", ".join(
+        inside_space_download_types))
     parser.add_argument("-nm", "--item_name", help="item name: e.g. project_name")
     parser.add_argument("-id", "--item_id", help="item id: e.g. Lifecycles-1")
     parser.add_argument("-bn", "--base_item_name", help="base item name, either local or on Octopus server")
@@ -179,6 +182,8 @@ def run():
         get_one_type_save(item_type=args.item_type, space_id=space_id)
     elif args.action == Actions.action_delete_type:
         delete_one_type(item_type=args.item_type, space_id=space_id)
+    elif args.action == Actions.action_delete_types:
+        delete_types(item_types_comma_delimited=args.item_types, space_id=space_id)
     elif args.action == Actions.action_get:
         get_single_item_by_name_or_id_save(item_type=args.item_type, item_name=args.item_name, item_id=args.item_id,
                                            space_id=space_id)
